@@ -1,16 +1,25 @@
 /*
- * "$Id: imagetops.c 7721 2008-07-11 22:48:49Z mike $"
+ * "$Id: imagetops.c,v 1.1.1.4 2002/06/06 22:13:02 jlovell Exp $"
  *
  *   Image file to PostScript filter for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007 by Apple Inc.
- *   Copyright 1993-2007 by Easy Software Products.
+ *   Copyright 1993-2002 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Apple Inc. and are protected by Federal copyright
- *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- *   which should have been included with this file.  If this file is
- *   file is missing or damaged, see the license at "http://www.cups.org/".
+ *   property of Easy Software Products and are protected by Federal
+ *   copyright law.  Distribution and use rights are outlined in the file
+ *   "LICENSE.txt" which should have been included with this file.  If this
+ *   file is missing or damaged please contact Easy Software Products
+ *   at:
+ *
+ *       Attn: CUPS Licensing Information
+ *       Easy Software Products
+ *       44141 Airport View Drive, Suite 204
+ *       Hollywood, Maryland 20636-3111 USA
+ *
+ *       Voice: (301) 373-9603
+ *       EMail: cups-info@cups.org
+ *         WWW: http://www.cups.org
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -28,7 +37,6 @@
 #include "common.h"
 #include "image.h"
 #include <math.h>
-#include <cups/i18n.h>
 
 
 /*
@@ -46,58 +54,57 @@ int	Flip = 0,		/* Flip/mirror pages */
  * Local functions...
  */
 
-static void	ps_hex(cups_ib_t *, int, int);
-static void	ps_ascii85(cups_ib_t *, int, int);
+static void	ps_hex(ib_t *, int, int);
+static void	ps_ascii85(ib_t *, int, int);
 
 
 /*
  * 'main()' - Main entry...
  */
 
-int					/* O - Exit status */
-main(int  argc,				/* I - Number of command-line arguments */
-     char *argv[])			/* I - Command-line arguments */
+int			/* O - Exit status */
+main(int  argc,		/* I - Number of command-line arguments */
+     char *argv[])	/* I - Command-line arguments */
 {
-  cups_image_t	*img;			/* Image to print */
-  float		xprint,			/* Printable area */
+  image_t	*img;		/* Image to print */
+  float		xprint,		/* Printable area */
 		yprint,
-		xinches,		/* Total size in inches */
+		xinches,	/* Total size in inches */
 		yinches;
-  float		xsize,			/* Total size in points */
+  float		xsize,		/* Total size in points */
 		ysize,
 		xsize2,
 		ysize2;
-  float		aspect;			/* Aspect ratio */
-  int		xpages,			/* # x pages */
-		ypages,			/* # y pages */
-		xpage,			/* Current x page */
-		ypage,			/* Current y page */
-		page;			/* Current page number */
-  int		xc0, yc0,			/* Corners of the page in image coords */
-		xc1, yc1;
-  cups_ib_t	*row;			/* Current row */
-  int		y;			/* Current Y coordinate in image */
-  int		colorspace;		/* Output colorspace */
-  int		out_offset,		/* Offset into output buffer */
-		out_length;		/* Length of output buffer */
-  ppd_file_t	*ppd;			/* PPD file */
-  ppd_choice_t	*choice;		/* PPD option choice */
-  int		num_options;		/* Number of print options */
-  cups_option_t	*options;		/* Print options */
-  const char	*val;			/* Option value */
-  int		slowcollate;		/* Collate copies the slow way */
-  float		g;			/* Gamma correction value */
-  float		b;			/* Brightness factor */
-  float		zoom;			/* Zoom facter */
-  int		xppi, yppi;		/* Pixels-per-inch */
-  int		hue, sat;		/* Hue and saturation adjustment */
-  int		realcopies,		/* Real copies being printed */
-		emit_jcl;		/* Emit JCL? */
-  float		left, top;		/* Left and top of image */
-  char		filename[1024];		/* Name of file to print */
-  time_t	curtime;		/* Current time */
-  struct tm	*curtm;			/* Current date */
-  char		curdate[255];		/* Current date string */
+  float		aspect;		/* Aspect ratio */
+  int		xpages,		/* # x pages */
+		ypages,		/* # y pages */
+		xpage,		/* Current x page */
+		ypage,		/* Current y page */
+		page;		/* Current page number */
+  int		x0, y0,		/* Corners of the page in image coords */
+		x1, y1;
+  ib_t		*row;		/* Current row */
+  int		y;		/* Current Y coordinate in image */
+  int		colorspace;	/* Output colorspace */
+  int		out_offset,	/* Offset into output buffer */
+		out_length;	/* Length of output buffer */
+  ppd_file_t	*ppd;		/* PPD file */
+  ppd_choice_t	*choice;	/* PPD option choice */
+  int		num_options;	/* Number of print options */
+  cups_option_t	*options;	/* Print options */
+  const char	*val;		/* Option value */
+  int		slowcollate;	/* Collate copies the slow way */
+  float		g;		/* Gamma correction value */
+  float		b;		/* Brightness factor */
+  float		zoom;		/* Zoom facter */
+  int		xppi, yppi;	/* Pixels-per-inch */
+  int		hue, sat;	/* Hue and saturation adjustment */
+  int		realcopies;	/* Real copies being printed */
+  float		left, top;	/* Left and top of image */
+  char		filename[1024];	/* Name of file to print */
+  time_t	curtime;	/* Current time */
+  struct tm	*curtm;		/* Current date */
+  char		curdate[255];	/* Current date string */
 
 
  /*
@@ -112,11 +119,13 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   if (argc < 6 || argc > 7)
   {
-    fprintf(stderr, _("Usage: %s job-id user title copies options [file]\n"),
-            argv[0]);
+    fputs("ERROR: imagetops job-id user title copies options [file]\n", stderr);
     return (1);
   }
 
+  fprintf(stderr, "INFO: %s %s %s %s %s %s %s\n", argv[0], argv[1], argv[2],
+          argv[3], argv[4], argv[5], argv[6] ? argv[6] : "(null)");
+
  /*
   * Copy stdin as needed...
   */
@@ -162,7 +171,7 @@ main(int  argc,				/* I - Number of command-line arguments */
   options     = NULL;
   num_options = cupsParseOptions(argv[5], 0, &options);
 
-  ppd = SetCommonOptions(num_options, options, 0);
+  ppd = SetCommonOptions(num_options, options, 1);
 
   if ((val = cupsGetOption("multiple-document-handling", num_options, options)) != NULL)
   {
@@ -183,37 +192,13 @@ main(int  argc,				/* I - Number of command-line arguments */
     Collate = 1;
 
   if ((val = cupsGetOption("gamma", num_options, options)) != NULL)
-  {
-   /*
-    * Get gamma value from 1 to 10000...
-    */
-
     g = atoi(val) * 0.001f;
 
-    if (g < 0.001f)
-      g = 0.001f;
-    else if (g > 10.0f)
-      g = 10.0f;
-  }
-
   if ((val = cupsGetOption("brightness", num_options, options)) != NULL)
-  {
-   /*
-    * Get brightness value from 10 to 1000.
-    */
-
     b = atoi(val) * 0.01f;
 
-    if (b < 0.1f)
-      b = 0.1f;
-    else if (b > 10.0f)
-      b = 10.0f;
-  }
-
   if ((val = cupsGetOption("scaling", num_options, options)) != NULL)
     zoom = atoi(val) * 0.01;
-  else if (cupsGetOption("fitplot", num_options, options))
-    zoom = 1.0;
 
   if ((val = cupsGetOption("ppi", num_options, options)) != NULL)
     if (sscanf(val, "%dx%d", &xppi, &yppi) < 2)
@@ -274,86 +259,50 @@ main(int  argc,				/* I - Number of command-line arguments */
   if ((val = cupsGetOption("hue", num_options, options)) != NULL)
     hue = atoi(val);
 
-  if ((choice = ppdFindMarkedChoice(ppd, "MirrorPrint")) != NULL)
-  {
-    val = choice->choice;
-    choice->marked = 0;
-  }
-  else
-    val = cupsGetOption("mirror", num_options, options);
-
-  if (val && (!strcasecmp(val, "true") || !strcasecmp(val, "on") ||
-              !strcasecmp(val, "yes")))
-    Flip = 1;
-
-  if ((val = cupsGetOption("emit-jcl", num_options, options)) != NULL &&
-      (!strcasecmp(val, "false") || !strcasecmp(val, "off") ||
-       !strcasecmp(val, "no") || !strcmp(val, "0")))
-    emit_jcl = 0;
-  else
-    emit_jcl = 1;
-
  /*
   * Open the input image to print...
   */
 
-  colorspace = ColorDevice ? CUPS_IMAGE_RGB_CMYK : CUPS_IMAGE_WHITE;
+  colorspace = ColorDevice ? IMAGE_RGB_CMYK : IMAGE_WHITE;
 
-  img = cupsImageOpen(filename, colorspace, CUPS_IMAGE_WHITE, sat, hue, NULL);
+  img = ImageOpen(filename, colorspace, IMAGE_WHITE, sat, hue, NULL);
 
   if (argc == 6)
     unlink(filename);
 
   if (img == NULL)
   {
-    fputs(_("ERROR: Unable to open image file for printing!\n"), stderr);
+    fputs("ERROR: Unable to open image file for printing!\n", stderr);
     ppdClose(ppd);
     return (1);
   }
 
-  colorspace = cupsImageGetColorSpace(img);
+  colorspace = img->colorspace;
 
  /*
   * Scale as necessary...
   */
 
+  xprint = (PageRight - PageLeft) / 72.0;
+  yprint = (PageTop - PageBottom) / 72.0;
+
   if (zoom == 0.0 && xppi == 0)
   {
-    xppi = cupsImageGetXPPI(img);
-    yppi = cupsImageGetYPPI(img);
+    xppi = img->xppi;
+    yppi = img->yppi;
   }
 
   if (yppi == 0)
     yppi = xppi;
 
-  fprintf(stderr, "DEBUG: Before scaling: xppi=%d, yppi=%d, zoom=%.2f\n",
-          xppi, yppi, zoom);
-
   if (xppi > 0)
   {
    /*
     * Scale the image as neccesary to match the desired pixels-per-inch.
     */
     
-    if (Orientation & 1)
-    {
-      xprint = (PageTop - PageBottom) / 72.0;
-      yprint = (PageRight - PageLeft) / 72.0;
-    }
-    else
-    {
-      xprint = (PageRight - PageLeft) / 72.0;
-      yprint = (PageTop - PageBottom) / 72.0;
-    }
-
-    fprintf(stderr, "DEBUG: Before scaling: xprint=%.1f, yprint=%.1f\n",
-            xprint, yprint);
-
-    xinches = (float)cupsImageGetWidth(img) / (float)xppi;
-    yinches = (float)cupsImageGetHeight(img) / (float)yppi;
-
-    fprintf(stderr, "DEBUG: Image size is %.1f x %.1f inches...\n",
-            xinches, yinches);
+    xinches = (float)img->xsize / (float)xppi;
+    yinches = (float)img->ysize / (float)yppi;
 
     if ((val = cupsGetOption("natural-scaling", num_options, options)) != NULL)
     {
@@ -368,8 +317,6 @@ main(int  argc,				/* I - Number of command-line arguments */
       * Rotate the image if it will fit landscape but not portrait...
       */
 
-      fputs("DEBUG: Auto orientation...\n", stderr);
-
       if ((xinches > xprint || yinches > yprint) &&
           xinches <= yprint && yinches <= xprint)
       {
@@ -377,12 +324,20 @@ main(int  argc,				/* I - Number of command-line arguments */
 	* Rotate the image as needed...
 	*/
 
-        fputs("DEBUG: Using landscape orientation...\n", stderr);
-
 	Orientation = (Orientation + 1) & 3;
 	xsize       = yprint;
 	yprint      = xprint;
 	xprint      = xsize;
+
+	xsize       = PageLeft;
+	PageLeft    = PageBottom;
+	PageBottom  = PageWidth - PageRight;
+	PageRight   = PageTop;
+	PageTop     = PageLength - xsize;
+
+	xsize       = PageWidth;
+	PageWidth   = PageLength;
+	PageLength  = xsize;
       }
     }
   }
@@ -392,36 +347,31 @@ main(int  argc,				/* I - Number of command-line arguments */
     * Scale percentage of page size...
     */
 
-    xprint = (PageRight - PageLeft) / 72.0;
-    yprint = (PageTop - PageBottom) / 72.0;
-    aspect = (float)cupsImageGetYPPI(img) / (float)cupsImageGetXPPI(img);
+    aspect = (float)img->yppi / (float)img->xppi;
 
-    fprintf(stderr, "DEBUG: Before scaling: xprint=%.1f, yprint=%.1f\n",
-            xprint, yprint);
-
-    fprintf(stderr, "DEBUG: cupsImageGetXPPI(img) = %d, cupsImageGetYPPI(img) = %d, aspect = %f\n",
-            cupsImageGetXPPI(img), cupsImageGetYPPI(img), aspect);
+    fprintf(stderr, "DEBUG: img->xppi = %d, img->yppi = %d, aspect = %f\n",
+            img->xppi, img->yppi, aspect);
 
     xsize = xprint * zoom;
-    ysize = xsize * cupsImageGetHeight(img) / cupsImageGetWidth(img) / aspect;
+    ysize = xsize * img->ysize / img->xsize / aspect;
 
     if (ysize > (yprint * zoom))
     {
       ysize = yprint * zoom;
-      xsize = ysize * cupsImageGetWidth(img) * aspect / cupsImageGetHeight(img);
+      xsize = ysize * img->xsize * aspect / img->ysize;
     }
 
     xsize2 = yprint * zoom;
-    ysize2 = xsize2 * cupsImageGetHeight(img) / cupsImageGetWidth(img) / aspect;
+    ysize2 = xsize2 * img->ysize / img->xsize / aspect;
 
     if (ysize2 > (xprint * zoom))
     {
       ysize2 = xprint * zoom;
-      xsize2 = ysize2 * cupsImageGetWidth(img) * aspect / cupsImageGetHeight(img);
+      xsize2 = ysize2 * img->xsize * aspect / img->ysize;
     }
 
-    fprintf(stderr, "DEBUG: Portrait size is %.2f x %.2f inches\n", xsize, ysize);
-    fprintf(stderr, "DEBUG: Landscape size is %.2f x %.2f inches\n", xsize2, ysize2);
+    fprintf(stderr, "DEBUG: xsize = %.0f, ysize = %.0f\n", xsize, ysize);
+    fprintf(stderr, "DEBUG: xsize2 = %.0f, ysize2 = %.0f\n", xsize2, ysize2);
 
     if (cupsGetOption("orientation-requested", num_options, options) == NULL &&
         cupsGetOption("landscape", num_options, options) == NULL)
@@ -431,21 +381,27 @@ main(int  argc,				/* I - Number of command-line arguments */
       * portrait if they are equal...
       */
 
-      fputs("DEBUG: Auto orientation...\n", stderr);
-
       if ((xsize * ysize) < (xsize2 * xsize2))
       {
        /*
 	* Do landscape orientation...
 	*/
 
-        fputs("DEBUG: Using landscape orientation...\n", stderr);
-
 	Orientation = 1;
 	xinches     = xsize2;
 	yinches     = ysize2;
 	xprint      = (PageTop - PageBottom) / 72.0;
 	yprint      = (PageRight - PageLeft) / 72.0;
+
+	xsize       = PageLeft;
+	PageLeft    = PageBottom;
+	PageBottom  = PageWidth - PageRight;
+	PageRight   = PageTop;
+	PageTop     = PageLength - xsize;
+
+	xsize       = PageWidth;
+	PageWidth   = PageLength;
+	PageLength  = xsize;
       }
       else
       {
@@ -453,8 +409,6 @@ main(int  argc,				/* I - Number of command-line arguments */
 	* Do portrait orientation...
 	*/
 
-        fputs("DEBUG: Using portrait orientation...\n", stderr);
-
 	Orientation = 0;
 	xinches     = xsize;
 	yinches     = ysize;
@@ -462,17 +416,23 @@ main(int  argc,				/* I - Number of command-line arguments */
     }
     else if (Orientation & 1)
     {
-      fputs("DEBUG: Using landscape orientation...\n", stderr);
-
       xinches     = xsize2;
       yinches     = ysize2;
       xprint      = (PageTop - PageBottom) / 72.0;
       yprint      = (PageRight - PageLeft) / 72.0;
+
+      xsize       = PageLeft;
+      PageLeft    = PageBottom;
+      PageBottom  = PageWidth - PageRight;
+      PageRight   = PageTop;
+      PageTop     = PageLength - xsize;
+
+      xsize       = PageWidth;
+      PageWidth   = PageLength;
+      PageLength  = xsize;
     }
     else
     {
-      fputs("DEBUG: Using portrait orientation...\n", stderr);
-
       xinches     = xsize;
       yinches     = ysize;
       xprint      = (PageRight - PageLeft) / 72.0;
@@ -491,9 +451,6 @@ main(int  argc,				/* I - Number of command-line arguments */
   xprint = xinches / xpages;
   yprint = yinches / ypages;
 
-  fprintf(stderr, "DEBUG: xpages = %dx%.2fin, ypages = %dx%.2fin\n",
-          xpages, xprint, ypages, yprint);
-
  /*
   * Update the page size for custom sizes...
   */
@@ -506,10 +463,6 @@ main(int  argc,				/* I - Number of command-line arguments */
     char	s[255];		/* New custom page size... */
 
 
-   /*
-    * Use the correct width and length for the current orientation...
-    */
-
     if (Orientation & 1)
     {
       width  = yprint * 72.0;
@@ -538,9 +491,6 @@ main(int  argc,				/* I - Number of command-line arguments */
     if (length < ppd->custom_min[1])
       length = ppd->custom_min[1];
 
-    fprintf(stderr, "DEBUG: Updated custom page size to %.2f x %.2f inches...\n",
-            width / 72.0, length / 72.0);
-
    /*
     * Set the new custom size...
     */
@@ -558,6 +508,8 @@ main(int  argc,				/* I - Number of command-line arguments */
     PageRight  = width - ppd->custom_margins[2];
     PageBottom = ppd->custom_margins[1];
     PageTop    = length - ppd->custom_margins[3];
+
+    UpdatePageVars();
   }
 
  /*
@@ -587,8 +539,7 @@ main(int  argc,				/* I - Number of command-line arguments */
   * Write any JCL commands that are needed to print PostScript code...
   */
 
-  if (emit_jcl)
-    ppdEmitJCL(ppd, stdout, atoi(argv[1]), argv[2], argv[3]);
+  ppdEmitJCL(ppd, stdout, atoi(argv[1]), argv[2], argv[3]);
 
  /*
   * Start sending the document with any commands needed...
@@ -605,14 +556,12 @@ main(int  argc,				/* I - Number of command-line arguments */
   puts("%%DocumentData: Clean7Bit");
   puts("%%DocumentNeededResources: font Helvetica-Bold");
   puts("%%Creator: imagetops/" CUPS_SVERSION);
-  strftime(curdate, sizeof(curdate), "%c", curtm);
+  strftime(curdate, sizeof(curdate), CUPS_STRFTIME_FORMAT, curtm);
   printf("%%%%CreationDate: %s\n", curdate);
-  WriteTextComment("Title", argv[3]);
-  WriteTextComment("For", argv[2]);
+  printf("%%%%Title: %s\n", argv[3]);
+  printf("%%%%For: %s\n", argv[2]);
   if (Orientation & 1)
     puts("%%Orientation: Landscape");
-  else
-    puts("%%Orientation: Portrait");
   puts("%%EndComments");
   puts("%%BeginProlog");
 
@@ -627,31 +576,8 @@ main(int  argc,				/* I - Number of command-line arguments */
     printf("{ neg 1 add dup 0 lt { pop 1 } { %.3f exp neg 1 add } "
            "ifelse %.3f mul } bind settransfer\n", g, b);
 
-  WriteCommon();
-  switch (Orientation)
-  {
-    case 0 :
-	WriteLabelProlog(cupsGetOption("page-label", num_options, options),
-                	 PageBottom, PageTop, PageWidth);
-        break;
-
-    case 1 :
-	WriteLabelProlog(cupsGetOption("page-label", num_options, options),
-                	 PageLeft, PageRight, PageLength);
-        break;
-
-    case 2 :
-	WriteLabelProlog(cupsGetOption("page-label", num_options, options),
-                	 PageLength - PageTop, PageLength - PageBottom,
-			 PageWidth);
-        break;
-
-    case 3 :
-	WriteLabelProlog(cupsGetOption("page-label", num_options, options),
-                	 PageWidth - PageRight, PageWidth - PageLeft,
-			 PageLength);
-        break;
-  }
+  WriteLabelProlog(cupsGetOption("page-label", num_options, options),
+                   PageBottom, PageTop, PageWidth);
 
   if (realcopies > 1)
   {
@@ -667,132 +593,7 @@ main(int  argc,				/* I - Number of command-line arguments */
   * Output the pages...
   */
 
-  row = malloc(cupsImageGetWidth(img) * abs(colorspace) + 3);
-
-  fprintf(stderr, "DEBUG: XPosition=%d, YPosition=%d, Orientation=%d\n",
-          XPosition, YPosition, Orientation);
-  fprintf(stderr, "DEBUG: xprint=%.0f, yprint=%.0f\n", xprint, yprint);
-  fprintf(stderr, "DEBUG: PageLeft=%.0f, PageRight=%.0f, PageWidth=%.0f\n",
-          PageLeft, PageRight, PageWidth);
-  fprintf(stderr, "DEBUG: PageBottom=%.0f, PageTop=%.0f, PageLength=%.0f\n",
-          PageBottom, PageTop, PageLength);
-
-  switch (Orientation)
-  {
-    default :
-	switch (XPosition)
-	{
-	  case -1 :
-              left = PageLeft;
-	      break;
-	  default :
-              left = (PageRight + PageLeft - xprint * 72) / 2;
-	      break;
-	  case 1 :
-              left = PageRight - xprint * 72;
-	      break;
-	}
-
-	switch (YPosition)
-	{
-	  case -1 :
-	      top = PageBottom + yprint * 72;
-	      break;
-	  default :
-	      top = (PageTop + PageBottom + yprint * 72) / 2;
-	      break;
-	  case 1 :
-	      top = PageTop;
-	      break;
-	}
-	break;
-
-    case 1 :
-	switch (XPosition)
-	{
-	  case -1 :
-              left = PageBottom;
-	      break;
-	  default :
-              left = (PageTop + PageBottom - xprint * 72) / 2;
-	      break;
-	  case 1 :
-              left = PageTop - xprint * 72;
-	      break;
-	}
-
-	switch (YPosition)
-	{
-	  case -1 :
-	      top = PageLeft + yprint * 72;
-	      break;
-	  default :
-	      top = (PageRight + PageLeft + yprint * 72) / 2;
-	      break;
-	  case 1 :
-	      top = PageRight;
-	      break;
-	}
-	break;
-
-    case 2 :
-	switch (XPosition)
-	{
-	  case 1 :
-              left = PageLeft;
-	      break;
-	  default :
-              left = (PageRight + PageLeft - xprint * 72) / 2;
-	      break;
-	  case -1 :
-              left = PageRight - xprint * 72;
-	      break;
-	}
-
-	switch (YPosition)
-	{
-	  case 1 :
-	      top = PageBottom + yprint * 72;
-	      break;
-	  default :
-	      top = (PageTop + PageBottom + yprint * 72) / 2;
-	      break;
-	  case -1 :
-	      top = PageTop;
-	      break;
-	}
-	break;
-
-    case 3 :
-	switch (XPosition)
-	{
-	  case 1 :
-              left = PageBottom;
-	      break;
-	  default :
-              left = (PageTop + PageBottom - xprint * 72) / 2;
-	      break;
-	  case -1 :
-              left = PageTop - xprint * 72;
-	      break;
-	}
-
-	switch (YPosition)
-	{
-	  case 1 :
-	      top = PageLeft + yprint * 72;
-	      break;
-	  default :
-	      top = (PageRight + PageLeft + yprint * 72) / 2;
-	      break;
-	  case -1 :
-	      top = PageRight;
-	      break;
-	}
-	break;
-  }
-
-  fprintf(stderr, "DEBUG: left=%.2f, top=%.2f\n", left, top);
+  row = malloc(img->xsize * abs(colorspace) + 3);
 
   for (page = 1; Copies > 0; Copies --)
     for (xpage = 0; xpage < xpages; xpage ++)
@@ -801,7 +602,7 @@ main(int  argc,				/* I - Number of command-line arguments */
         if (ppd && ppd->num_filters == 0)
           fprintf(stderr, "PAGE: %d %d\n", page, realcopies);
 
-	fprintf(stderr, _("INFO: Printing page %d...\n"), page);
+	fprintf(stderr, "INFO: Printing page %d...\n", page);
 
         printf("%%%%Page: %d %d\n", page, page);
 
@@ -815,57 +616,81 @@ main(int  argc,				/* I - Number of command-line arguments */
 	switch (Orientation)
 	{
 	  case 1 : /* Landscape */
-              printf("%.0f 0 translate 90 rotate\n", PageWidth);
+              printf("%.0f 0 translate 90 rotate\n", PageLength);
               break;
 	  case 2 : /* Reverse Portrait */
               printf("%.0f %.0f translate 180 rotate\n", PageWidth, PageLength);
               break;
 	  case 3 : /* Reverse Landscape */
-              printf("0 %.0f translate -90 rotate\n", PageLength);
+              printf("0 %.0f translate -90 rotate\n", PageWidth);
               break;
 	}
 
-        puts("gsave");
+	x0 = img->xsize * xpage / xpages;
+	x1 = img->xsize * (xpage + 1) / xpages - 1;
+	y0 = img->ysize * ypage / ypages;
+	y1 = img->ysize * (ypage + 1) / ypages - 1;
 
-	xc0 = cupsImageGetWidth(img) * xpage / xpages;
-	xc1 = cupsImageGetWidth(img) * (xpage + 1) / xpages - 1;
-	yc0 = cupsImageGetHeight(img) * ypage / ypages;
-	yc1 = cupsImageGetHeight(img) * (ypage + 1) / ypages - 1;
+        switch (XPosition)
+	{
+	  case -1 :
+	      left = PageLeft;
+	      break;
+	  default :
+	      left = (PageWidth - xprint * 72.0) * 0.5;
+	      break;
+	  case 1 :
+	      left = PageRight - xprint * 72.0;
+	      break;
+	}
+
+        switch (YPosition)
+	{
+	  case -1 :
+	      top = PageBottom + 72.0 * yprint;
+	      break;
+	  default :
+	      top = (PageLength + yprint * 72.0) * 0.5;
+	      break;
+	  case 1 :
+	      top = PageTop;
+	      break;
+	}
 
         printf("%.1f %.1f translate\n", left, top);
 
 	printf("%.3f %.3f scale\n\n",
-	       xprint * 72.0 / (xc1 - xc0 + 1),
-	       yprint * 72.0 / (yc1 - yc0 + 1));
+	       xprint * 72.0 / (x1 - x0 + 1),
+	       yprint * 72.0 / (y1 - y0 + 1));
 
 	if (LanguageLevel == 1)
 	{
-	  printf("/picture %d string def\n", (xc1 - xc0 + 1) * abs(colorspace));
-	  printf("%d %d 8[1 0 0 -1 0 1]", (xc1 - xc0 + 1), (yc1 - yc0 + 1));
+	  printf("/picture %d string def\n", (x1 - x0 + 1) * abs(colorspace));
+	  printf("%d %d 8[1 0 0 -1 0 1]", (x1 - x0 + 1), (y1 - y0 + 1));
 
-          if (colorspace == CUPS_IMAGE_WHITE)
+          if (colorspace == IMAGE_WHITE)
             puts("{currentfile picture readhexstring pop} image");
           else
             printf("{currentfile picture readhexstring pop} false %d colorimage\n",
 	           abs(colorspace));
 
-          for (y = yc0; y <= yc1; y ++)
+          for (y = y0; y <= y1; y ++)
           {
-            cupsImageGetRow(img, xc0, y, xc1 - xc0 + 1, row);
-            ps_hex(row, (xc1 - xc0 + 1) * abs(colorspace), y == yc1);
+            ImageGetRow(img, x0, y, x1 - x0 + 1, row);
+            ps_hex(row, (x1 - x0 + 1) * abs(colorspace), y == y1);
           }
 	}
 	else
 	{
           switch (colorspace)
 	  {
-	    case CUPS_IMAGE_WHITE :
+	    case IMAGE_WHITE :
                 puts("/DeviceGray setcolorspace");
 		break;
-            case CUPS_IMAGE_RGB :
+            case IMAGE_RGB :
                 puts("/DeviceRGB setcolorspace");
 		break;
-            case CUPS_IMAGE_CMYK :
+            case IMAGE_CMYK :
                 puts("/DeviceCMYK setcolorspace");
 		break;
           }
@@ -875,36 +700,36 @@ main(int  argc,				/* I - Number of command-line arguments */
 		 "/Width %d"
 		 "/Height %d"
 		 "/BitsPerComponent 8",
-		 xc1 - xc0 + 1, yc1 - yc0 + 1);
+		 x1 - x0 + 1, y1 - y0 + 1);
 
           switch (colorspace)
 	  {
-	    case CUPS_IMAGE_WHITE :
+	    case IMAGE_WHITE :
                 fputs("/Decode[0 1]", stdout);
 		break;
-            case CUPS_IMAGE_RGB :
+            case IMAGE_RGB :
                 fputs("/Decode[0 1 0 1 0 1]", stdout);
 		break;
-            case CUPS_IMAGE_CMYK :
+            case IMAGE_CMYK :
                 fputs("/Decode[0 1 0 1 0 1 0 1]", stdout);
 		break;
           }
 
-          fputs("\n/DataSource currentfile/ASCII85Decode filter", stdout);
+          fputs("/DataSource currentfile /ASCII85Decode filter", stdout);
 
-          if (((xc1 - xc0 + 1) / xprint) < 100.0)
+          if (((x1 - x0 + 1) / xprint) < 100.0)
             fputs("/Interpolate true", stdout);
 
           puts("/ImageMatrix[1 0 0 -1 0 1]>>image");
 
-          for (y = yc0, out_offset = 0; y <= yc1; y ++)
+          for (y = y0, out_offset = 0; y <= y1; y ++)
           {
-            cupsImageGetRow(img, xc0, y, xc1 - xc0 + 1, row + out_offset);
+            ImageGetRow(img, x0, y, x1 - x0 + 1, row + out_offset);
 
-            out_length = (xc1 - xc0 + 1) * abs(colorspace) + out_offset;
+            out_length = (x1 - x0 + 1) * abs(colorspace) + out_offset;
             out_offset = out_length & 3;
 
-            ps_ascii85(row, out_length, y == yc1);
+            ps_ascii85(row, out_length, y == y1);
 
             if (out_offset > 0)
               memcpy(row, row + out_length - out_offset, out_offset);
@@ -912,8 +737,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	}
 
 	puts("grestore");
-	WriteLabels(0);
-	puts("grestore");
+	WriteLabels(Orientation);
 	puts("showpage");
       }
 
@@ -923,19 +747,16 @@ main(int  argc,				/* I - Number of command-line arguments */
   * End the job with the appropriate JCL command or CTRL-D otherwise.
   */
 
-  if (emit_jcl)
-  {
-    if (ppd && ppd->jcl_end)
-      ppdEmitJCLEnd(ppd, stdout);
-    else
-      putchar(0x04);
-  }
+  if (ppd != NULL && ppd->jcl_end)
+    fputs(ppd->jcl_end, stdout);
+  else
+    putchar(0x04);
 
  /*
   * Close files...
   */
 
-  cupsImageClose(img);
+  ImageClose(img);
   ppdClose(ppd);
 
   return (0);
@@ -947,13 +768,13 @@ main(int  argc,				/* I - Number of command-line arguments */
  */
 
 static void
-ps_hex(cups_ib_t *data,			/* I - Data to print */
-       int       length,		/* I - Number of bytes to print */
-       int       last_line)		/* I - Last line of raster data? */
+ps_hex(ib_t *data,		/* I - Data to print */
+       int  length,		/* I - Number of bytes to print */
+       int  last_line)		/* I - Last line of raster data? */
 {
-  static int	col = 0;		/* Current column */
+  static int	col = 0;	/* Current column */
   static char	*hex = "0123456789ABCDEF";
-					/* Hex digits */
+				/* Hex digits */
 
 
   while (length > 0)
@@ -990,13 +811,13 @@ ps_hex(cups_ib_t *data,			/* I - Data to print */
  */
 
 static void
-ps_ascii85(cups_ib_t *data,		/* I - Data to print */
-	   int       length,		/* I - Number of bytes to print */
-	   int       last_line)		/* I - Last line of raster data? */
+ps_ascii85(ib_t *data,		/* I - Data to print */
+	   int  length,		/* I - Number of bytes to print */
+	   int  last_line)	/* I - Last line of raster data? */
 {
-  unsigned	b;			/* Binary data word */
-  unsigned char	c[5];			/* ASCII85 encoded chars */
-  static int	col = 0;		/* Current column */
+  unsigned	b;		/* Binary data word */
+  unsigned char	c[5];		/* ASCII85 encoded chars */
+  static int	col = 0;	/* Current column */
 
 
   while (length > 3)
@@ -1061,5 +882,5 @@ ps_ascii85(cups_ib_t *data,		/* I - Data to print */
 
 
 /*
- * End of "$Id: imagetops.c 7721 2008-07-11 22:48:49Z mike $".
+ * End of "$Id: imagetops.c,v 1.1.1.4 2002/06/06 22:13:02 jlovell Exp $".
  */
