@@ -1,12 +1,38 @@
 /*
- * EPSON ESC/P and ESC/P2 filter for CUPS.
+ * "$Id: rastertoepson.c,v 1.1.1.3 2002/03/02 18:28:31 jlovell Exp $"
  *
- * Copyright © 2020-2024 by OpenPrinting.
- * Copyright 2007-2018 by Apple Inc.
- * Copyright 1993-2007 by Easy Software Products.
+ *   EPSON ESC/P and ESC/P2 filter for the Common UNIX Printing System
+ *   (CUPS).
  *
- * Licensed under Apache License v2.0.  See the file "LICENSE" for more
- * information.
+ *   Copyright 1993-2002 by Easy Software Products.
+ *
+ *   These coded instructions, statements, and computer programs are the
+ *   property of Easy Software Products and are protected by Federal
+ *   copyright law.  Distribution and use rights are outlined in the file
+ *   "LICENSE.txt" which should have been included with this file.  If this
+ *   file is missing or damaged please contact Easy Software Products
+ *   at:
+ *
+ *       Attn: CUPS Licensing Information
+ *       Easy Software Products
+ *       44141 Airport View Drive, Suite 204
+ *       Hollywood, Maryland 20636-3111 USA
+ *
+ *       Voice: (301) 373-9603
+ *       EMail: cups-info@cups.org
+ *         WWW: http://www.cups.org
+ *
+ *   This file is subject to the Apple OS-Developed Software exception.
+ *
+ * Contents:
+ *
+ *   Setup()        - Prepare the printer for printing.
+ *   StartPage()    - Start a page of graphics.
+ *   EndPage()      - Finish a page of graphics.
+ *   Shutdown()     - Shutdown the printer.
+ *   CompressData() - Compress a line of graphics.
+ *   OutputLine()   - Output a line of graphics.
+ *   main()         - Main entry and processing of driver.
  */
 
 /*
@@ -15,9 +41,9 @@
 
 #include <cups/cups.h>
 #include <cups/ppd.h>
-#include <cups/string-private.h>
-#include <cups/language-private.h>
-#include <cups/raster.h>
+#include <cups/string.h>
+#include "raster.h"
+#include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <signal.h>
@@ -50,17 +76,15 @@ unsigned char	*Planes[6],		/* Output buffers */
 		*CompBuffer,		/* Compression buffer */
 		*LineBuffers[2];	/* Line bitmap buffers */
 int		Model,			/* Model number */
-		EjectPage,		/* Eject the page when done? */
-		Shingling,		/* Shingle output? */
-		Canceled;		/* Has the current job been canceled? */
-unsigned	NumPlanes,		/* Number of color planes */
-		Feed,			/* Number of lines to skip */
-		DotBit,			/* Bit in buffers */
+		NumPlanes,		/* Number of color planes */
+		Feed;			/* Number of lines to skip */
+int		DotBit,			/* Bit in buffers */
 		DotBytes,		/* # bytes in a dot column */
 		DotColumns,		/* # columns in 1/60 inch */
 		LineCount,		/* # of lines processed */
 		EvenOffset,		/* Offset into 'even' buffers */
-		OddOffset;		/* Offset into 'odd' buffers */
+		OddOffset,		/* Offset into 'odd' buffers */
+		Shingling;		/* Shingle output? */
 
 
 /*
@@ -68,15 +92,15 @@ unsigned	NumPlanes,		/* Number of color planes */
  */
 
 void	Setup(void);
-void	StartPage(const ppd_file_t *ppd, const cups_page_header2_t *header);
-void	EndPage(const cups_page_header2_t *header);
+void	StartPage(const ppd_file_t *ppd, const cups_page_header_t *header);
+void	EndPage(const cups_page_header_t *header);
 void	Shutdown(void);
 
 void	CancelJob(int sig);
-void	CompressData(const unsigned char *line, unsigned length, unsigned plane,
-	             unsigned type, unsigned xstep, unsigned ystep);
-void	OutputLine(const cups_page_header2_t *header);
-void	OutputRows(const cups_page_header2_t *header, int row);
+void	CompressData(const unsigned char *line, int length, int plane,
+	             int type, int xstep, int ystep);
+void	OutputLine(const cups_page_header_t *header);
+void	OutputRows(const cups_page_header_t *header, int row);
 
 
 /*
@@ -86,7 +110,7 @@ void	OutputRows(const cups_page_header2_t *header, int row);
 void
 Setup(void)
 {
-  const char	*device_uri;		/* The device for the printer... */
+  const char	*device_uri;	/* The device for the printer... */
 
 
  /*
@@ -95,7 +119,7 @@ Setup(void)
   */
 
   if ((device_uri = getenv("DEVICE_URI")) != NULL &&
-      strncmp(device_uri, "usb:", 4) == 0 && Model >= EPSON_ICOLOR)
+      strncmp(device_uri, "usb:", 4) == 0)
     pwrite("\000\000\000\033\001@EJL 1284.4\n@EJL     \n\033@", 29);
 }
 
@@ -105,43 +129,38 @@ Setup(void)
  */
 
 void
-StartPage(
-    const ppd_file_t         *ppd,	/* I - PPD file */
-    const cups_page_header2_t *header)	/* I - Page header */
+StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
+          const cups_page_header_t *header)	/* I - Page header */
 {
-  int		n, t;			/* Numbers */
-  unsigned	plane;			/* Looping var */
+  int	n, t;					/* Numbers */
+  int	plane;					/* Looping var */
+#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
+  struct sigaction action;			/* Actions for POSIX signals */
+#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
  /*
-  * Show page device dictionary...
+  * Register a signal handler to eject the current page if the
+  * job is cancelled.
   */
 
-  fprintf(stderr, "DEBUG: StartPage...\n");
-  fprintf(stderr, "DEBUG: Duplex = %d\n", header->Duplex);
-  fprintf(stderr, "DEBUG: HWResolution = [ %d %d ]\n", header->HWResolution[0], header->HWResolution[1]);
-  fprintf(stderr, "DEBUG: ImagingBoundingBox = [ %d %d %d %d ]\n", header->ImagingBoundingBox[0], header->ImagingBoundingBox[1], header->ImagingBoundingBox[2], header->ImagingBoundingBox[3]);
-  fprintf(stderr, "DEBUG: Margins = [ %d %d ]\n", header->Margins[0], header->Margins[1]);
-  fprintf(stderr, "DEBUG: ManualFeed = %d\n", header->ManualFeed);
-  fprintf(stderr, "DEBUG: MediaPosition = %d\n", header->MediaPosition);
-  fprintf(stderr, "DEBUG: NumCopies = %d\n", header->NumCopies);
-  fprintf(stderr, "DEBUG: Orientation = %d\n", header->Orientation);
-  fprintf(stderr, "DEBUG: PageSize = [ %d %d ]\n", header->PageSize[0], header->PageSize[1]);
-  fprintf(stderr, "DEBUG: cupsWidth = %d\n", header->cupsWidth);
-  fprintf(stderr, "DEBUG: cupsHeight = %d\n", header->cupsHeight);
-  fprintf(stderr, "DEBUG: cupsMediaType = %d\n", header->cupsMediaType);
-  fprintf(stderr, "DEBUG: cupsBitsPerColor = %d\n", header->cupsBitsPerColor);
-  fprintf(stderr, "DEBUG: cupsBitsPerPixel = %d\n", header->cupsBitsPerPixel);
-  fprintf(stderr, "DEBUG: cupsBytesPerLine = %d\n", header->cupsBytesPerLine);
-  fprintf(stderr, "DEBUG: cupsColorOrder = %d\n", header->cupsColorOrder);
-  fprintf(stderr, "DEBUG: cupsColorSpace = %d\n", header->cupsColorSpace);
-  fprintf(stderr, "DEBUG: cupsCompression = %d\n", header->cupsCompression);
+#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
+  sigset(SIGTERM, CancelJob);
+#elif defined(HAVE_SIGACTION)
+  memset(&action, 0, sizeof(action));
+
+  sigemptyset(&action.sa_mask);
+  action.sa_handler = CancelJob;
+  sigaction(SIGTERM, &action, NULL);
+#else
+  signal(SIGTERM, CancelJob);
+#endif /* HAVE_SIGSET */
 
  /*
   * Send a reset sequence.
   */
 
-  if (ppd && ppd->nickname && strstr(ppd->nickname, "OKIDATA") != NULL)
+  if (ppd->nickname && strstr(ppd->nickname, "OKIDATA") != NULL)
     printf("\033{A");	/* Set EPSON emulation mode */
 
   printf("\033@");
@@ -150,11 +169,13 @@ StartPage(
   * See which type of printer we are using...
   */
 
-  switch (Model)
+  Model = ppd->model_number;
+
+  switch (ppd->model_number)
   {
     case EPSON_9PIN :
     case EPSON_24PIN :
-        printf("\033P\022");		/* Set 10 CPI */
+        printf("\033P");		/* Set 10 CPI */
 
 	if (header->HWResolution[0] == 360 || header->HWResolution[0] == 240)
 	{
@@ -169,10 +190,9 @@ StartPage(
 
 	printf("\033l%c\033Q%c", 0,	/* Side margins */
                       (int)(10.0 * header->PageSize[0] / 72.0 + 0.5));
-	printf("\033\062\033C%c",	/* Page length in 1/6th inches */
-		      (int)(header->PageSize[1] / 12.0 + 0.5));
+	printf("\033C%c%c", 0,		/* Page length */
+                      (int)(header->PageSize[1] / 72.0 + 0.5));
 	printf("\033N%c", 0);		/* Bottom margin */
-        printf("\033O");		/* No perforation skip */
 
        /*
 	* Setup various buffer limits...
@@ -182,14 +202,13 @@ StartPage(
 	DotColumns = header->HWResolution[0] / 60;
         Shingling  = 0;
 
-        if (Model == EPSON_9PIN)
+        if (ppd->model_number == EPSON_9PIN)
 	  printf("\033\063\030");	/* Set line feed */
 	else
 	  switch (header->HWResolution[0])
 	  {
 	    case 60:
 	    case 120 :
-	    case 240 :
         	printf("\033\063\030");	/* Set line feed */
 		break;
 
@@ -219,28 +238,26 @@ StartPage(
         if (Model < EPSON_ICOLOR)
 	{
 	  pwrite("\033(U\001\000", 5);		/* Resolution/units */
-	  putchar((int)(3600 / header->HWResolution[1]));
+	  putchar(3600 / header->HWResolution[1]);
         }
 	else
 	{
 	  pwrite("\033(U\005\000", 5);
-	  putchar((int)(1440 / header->HWResolution[1]));
-	  putchar((int)(1440 / header->HWResolution[1]));
-	  putchar((int)(1440 / header->HWResolution[0]));
+	  putchar(1440 / header->HWResolution[1]);
+	  putchar(1440 / header->HWResolution[1]);
+	  putchar(1440 / header->HWResolution[0]);
 	  putchar(0xa0);	/* n/1440ths... */
 	  putchar(0x05);
 	}
 
-	n = (int)(header->PageSize[1] * header->HWResolution[1] / 72.0);
+	n = header->PageSize[1] * header->HWResolution[1] / 72.0;
 
 	pwrite("\033(C\002\000", 5);		/* Page length */
 	putchar(n);
 	putchar(n >> 8);
 
-        if (ppd)
-	  t = (int)((ppd->sizes[1].length - ppd->sizes[1].top) * header->HWResolution[1] / 72.0);
-        else
-	  t = 0;
+	t = (ppd->sizes[1].length - ppd->sizes[1].top) *
+	    header->HWResolution[1] / 72.0;
 
 	pwrite("\033(c\004\000", 5);		/* Top & bottom margins */
 	putchar(t);
@@ -281,34 +298,18 @@ StartPage(
   * Allocate memory for a line/row of graphics...
   */
 
-  if ((Planes[0] = malloc(header->cupsBytesPerLine + NumPlanes)) == NULL)
-  {
-    fputs("ERROR: Unable to allocate memory\n", stderr);
-    exit(1);
-  }
-
+  Planes[0] = malloc(header->cupsBytesPerLine);
   for (plane = 1; plane < NumPlanes; plane ++)
     Planes[plane] = Planes[0] + plane * header->cupsBytesPerLine / NumPlanes;
 
   if (header->cupsCompression || DotBytes)
-  {
-    if ((CompBuffer = calloc(2, header->cupsWidth + 1)) == NULL)
-    {
-      fputs("ERROR: Unable to allocate memory\n", stderr);
-      exit(1);
-    }
-  }
+    CompBuffer = calloc(2, header->cupsWidth);
   else
     CompBuffer = NULL;
 
   if (DotBytes)
   {
-    if ((LineBuffers[0] = calloc((size_t)DotBytes, (header->cupsWidth + 7) * (size_t)(Shingling + 1))) == NULL)
-    {
-      fputs("ERROR: Unable to allocate memory\n", stderr);
-      exit(1);
-    }
-
+    LineBuffers[0] = calloc(DotBytes, header->cupsWidth * (Shingling + 1));
     LineBuffers[1] = LineBuffers[0] + DotBytes * header->cupsWidth;
     DotBit         = 128;
     LineCount      = 0;
@@ -323,9 +324,13 @@ StartPage(
  */
 
 void
-EndPage(
-    const cups_page_header2_t *header)	/* I - Page header */
+EndPage(const cups_page_header_t *header)	/* I - Page header */
 {
+#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
+  struct sigaction action;			/* Actions for POSIX signals */
+#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
+
+
   if (DotBytes && header)
   {
    /*
@@ -333,10 +338,7 @@ EndPage(
     */
 
     if (!Shingling)
-    {
-      if (DotBit < 128 || EvenOffset)
-        OutputRows(header, 0);
-    }
+      OutputRows(header, 0);
     else if (OddOffset > EvenOffset)
     {
       OutputRows(header, 1);
@@ -353,9 +355,25 @@ EndPage(
   * Eject the current page...
   */
 
-  putchar(12);				/* Form feed */
+  putchar(12);		/* Form feed */
   fflush(stdout);
 
+ /*
+  * Unregister the signal handler...
+  */
+
+#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
+  sigset(SIGTERM, SIG_IGN);
+#elif defined(HAVE_SIGACTION)
+  memset(&action, 0, sizeof(action));
+
+  sigemptyset(&action.sa_mask);
+  action.sa_handler = SIG_IGN;
+  sigaction(SIGTERM, &action, NULL);
+#else
+  signal(SIGTERM, SIG_IGN);
+#endif /* HAVE_SIGSET */
+
  /*
   * Free memory...
   */
@@ -392,9 +410,31 @@ Shutdown(void)
 void
 CancelJob(int sig)			/* I - Signal */
 {
+  int	i;				/* Looping var */
+
+
   (void)sig;
 
-  Canceled = 1;
+ /*
+  * Send out lots of NUL bytes to clear out any pending raster data...
+  */
+
+  if (DotBytes)
+    i = DotBytes * 360 * 8;
+  else
+    i = 720;
+
+  for (; i > 0; i --)
+    putchar(0);
+
+ /*
+  * End the current page and exit...
+  */
+
+  EndPage(NULL);
+  Shutdown();
+
+  exit(0);
 }
 
 
@@ -404,11 +444,11 @@ CancelJob(int sig)			/* I - Signal */
 
 void
 CompressData(const unsigned char *line,	/* I - Data to compress */
-             unsigned            length,/* I - Number of bytes */
-	     unsigned            plane,	/* I - Color plane */
-	     unsigned            type,	/* I - Type of compression */
-	     unsigned            xstep,	/* I - X resolution */
-	     unsigned            ystep)	/* I - Y resolution */
+             int                 length,/* I - Number of bytes */
+	     int                 plane,	/* I - Color plane */
+	     int                 type,	/* I - Type of compression */
+	     int                 xstep,	/* I - X resolution */
+	     int                 ystep)	/* I - Y resolution */
 {
   const unsigned char	*line_ptr,	/* Current byte pointer */
         		*line_end,	/* End-of-line byte pointer */
@@ -463,10 +503,11 @@ CompressData(const unsigned char *line,	/* I - Data to compress */
       *comp_ptr++ = temp;
 
      /*
-      * Check the last bit in the current byte...
+      * Check the last bit in the current byte and the first bit in the
+      * next byte...
       */
 
-      if ((temp & 0x01) && comp_ptr < line_end)
+      if ((temp & 0x01) && comp_ptr < line_end && *comp_ptr & 0x80)
         *comp_ptr &= 0x7f;
     }
   }
@@ -514,7 +555,7 @@ CompressData(const unsigned char *line,	/* I - Data to compress */
               count ++;
 	    }
 
-	    *comp_ptr++ = (unsigned char)(257 - count);
+	    *comp_ptr++ = 257 - count;
 	    *comp_ptr++ = *line_ptr++;
 	  }
 	  else
@@ -535,9 +576,9 @@ CompressData(const unsigned char *line,	/* I - Data to compress */
               count ++;
 	    }
 
-	    *comp_ptr++ = (unsigned char)(count - 1);
+	    *comp_ptr++ = count - 1;
 
-	    memcpy(comp_ptr, start, (size_t)count);
+	    memcpy(comp_ptr, start, count);
 	    comp_ptr += count;
 	  }
 	}
@@ -577,12 +618,12 @@ CompressData(const unsigned char *line,	/* I - Data to compress */
 
     length *= 8;
     printf("\033.");			/* Raster graphics */
-    putchar((int)type);
-    putchar((int)ystep);
-    putchar((int)xstep);
+    putchar(type);
+    putchar(ystep);
+    putchar(xstep);
     putchar(1);
-    putchar((int)length);
-    putchar((int)(length >> 8));
+    putchar(length);
+    putchar(length >> 8);
   }
   else
   {
@@ -592,15 +633,15 @@ CompressData(const unsigned char *line,	/* I - Data to compress */
 
     printf("\033i");
     putchar(ctable[plane]);
-    putchar((int)type);
+    putchar(type);
     putchar(1);
-    putchar((int)length);
-    putchar((int)(length >> 8));
+    putchar(length & 255);
+    putchar(length >> 8);
     putchar(1);
     putchar(0);
   }
 
-  pwrite(line_ptr, (size_t)(line_end - line_ptr));
+  pwrite(line_ptr, line_end - line_ptr);
   fflush(stdout);
 }
 
@@ -610,16 +651,15 @@ CompressData(const unsigned char *line,	/* I - Data to compress */
  */
 
 void
-OutputLine(
-    const cups_page_header2_t *header)	/* I - Page header */
+OutputLine(const cups_page_header_t *header)	/* I - Page header */
 {
   if (header->cupsRowCount)
   {
-    unsigned		width;
+    int			width;
     unsigned char	*tempptr,
 			*evenptr,
 			*oddptr;
-    unsigned int	x;
+    register int	x;
     unsigned char	bit;
     const unsigned char	*pixel;
     unsigned char 	*temp;
@@ -672,19 +712,13 @@ OutputLine(
         }
 
         for (width = header->cupsWidth, tempptr = CompBuffer;
-             width > 1;
+             width > 0;
              width -= 2, tempptr += 2, oddptr += DotBytes * 2,
 	         evenptr += DotBytes * 2)
         {
           evenptr[0] = tempptr[0];
           oddptr[0]  = tempptr[1];
         }
-
-        if (width == 1)
-        {
-          evenptr[0] = tempptr[0];
-          oddptr[0]  = tempptr[1];
-        }
       }
       else
       {
@@ -694,7 +728,7 @@ OutputLine(
 
         for (width = header->cupsWidth, tempptr = CompBuffer,
                  evenptr = LineBuffers[0] + EvenOffset;
-             width > 0;
+             width >= 0;
              width --, tempptr ++, evenptr += DotBytes)
           *evenptr = tempptr[0];
       }
@@ -735,9 +769,10 @@ OutputLine(
   }
   else
   {
-    unsigned	plane;		/* Current plane */
-    unsigned	bytes;		/* Bytes per plane */
-    unsigned	xstep, ystep;	/* X & Y resolutions */
+    int	plane;		/* Current plane */
+    int	bytes;		/* Bytes per plane */
+    int	xstep, ystep;	/* X & Y resolutions */
+
 
    /*
     * Write a single line of bitmap data as needed...
@@ -754,7 +789,7 @@ OutputLine(
       */
 
       if (!Planes[plane][0] &&
-          memcmp(Planes[plane], Planes[plane] + 1, (size_t)bytes - 1) == 0)
+          memcmp(Planes[plane], Planes[plane] + 1, bytes - 1) == 0)
 	continue;
 
      /*
@@ -764,13 +799,14 @@ OutputLine(
       if (Feed > 0)
       {
 	pwrite("\033(v\002\000", 5);	/* Relative vertical position */
-	putchar((int)Feed);
-	putchar((int)(Feed >> 8));
+	putchar(Feed);
+	putchar(Feed >> 8);
 
 	Feed = 0;
       }
 
-      CompressData(Planes[plane], bytes, plane, header->cupsCompression, xstep, ystep);
+      CompressData(Planes[plane], bytes, plane, header->cupsCompression, xstep,
+                   ystep);
     }
 
     Feed ++;
@@ -783,21 +819,21 @@ OutputLine(
  */
 
 void
-OutputRows(
-    const cups_page_header2_t *header,	/* I - Page image header */
-    int                      row)	/* I - Row number (0 or 1) */
+OutputRows(const cups_page_header_t *header,	/* I - Page image header */
+           int                      row)	/* I - Row number (0 or 1) */
 {
-  unsigned	i, n,			/* Looping vars */
-		dot_count,		/* Number of bytes to print */
-                dot_min;		/* Minimum number of bytes */
-  unsigned char *dot_ptr,		/* Pointer to print data */
-		*ptr;			/* Current data */
+  unsigned	i, n;				/* Looping vars */
+  int		dot_count,			/* Number of bytes to print */
+                dot_min;			/* Minimum number of bytes */
+  unsigned char *dot_ptr,			/* Pointer to print data */
+		*ptr;				/* Current data */
 
 
   dot_min = DotBytes * DotColumns;
 
   if (LineBuffers[row][0] != 0 ||
-      memcmp(LineBuffers[row], LineBuffers[row] + 1, header->cupsWidth * DotBytes - 1))
+      memcmp(LineBuffers[row], LineBuffers[row] + 1,
+             header->cupsWidth * DotBytes - 1))
   {
    /*
     * Skip leading space...
@@ -828,15 +864,10 @@ OutputRows(
     * Position print head for printing...
     */
 
-    if (i == 0)
-      putchar('\r');
-    else
-    {
-      putchar(0x1b);
-      putchar('$');
-      putchar((int)(i & 255));
-      putchar((int)(i >> 8));
-    }
+    putchar(0x1b);
+    putchar('$');
+    putchar(i & 255);
+    putchar(i >> 8);
 
    /*
     * Start bitmap graphics for this line...
@@ -875,9 +906,9 @@ OutputRows(
           break;
     }
 
-    n = dot_count / DotBytes;
-    putchar((int)(n & 255));
-    putchar((int)(n / 256));
+    n = (unsigned)dot_count / DotBytes;
+    putchar(n & 255);
+    putchar(n / 256);
 
    /*
     * Write the graphics data...
@@ -896,22 +927,14 @@ OutputRows(
 	putchar(0);
       }
 
-      if (dot_count & 1)
-        putchar(*ptr);
-
      /*
       * Move the head back and print the odd bytes...
       */
 
-      if (i == 0)
-	putchar('\r');
-      else
-      {
-	putchar(0x1b);
-	putchar('$');
-	putchar((int)(i & 255));
-	putchar((int)(i >> 8));
-      }
+      putchar(0x1b);
+      putchar('$');
+      putchar(i & 255);
+      putchar(i >> 8);
 
       if (header->HWResolution[0] == 120)
       	printf("\033*\001");		/* Select bit image */
@@ -919,17 +942,14 @@ OutputRows(
       	printf("\033*\003");		/* Select bit image */
 
       n = (unsigned)dot_count / DotBytes;
-      putchar((int)(n & 255));
-      putchar((int)(n / 256));
+      putchar(n & 255);
+      putchar(n / 256);
 
       for (n = dot_count / 2, ptr = dot_ptr + 1; n > 0; n --, ptr += 2)
       {
 	putchar(0);
         putchar(*ptr);
       }
-
-      if (dot_count & 1)
-        putchar(0);
     }
     else
       pwrite(dot_ptr, dot_count);
@@ -955,7 +975,7 @@ OutputRows(
   * Clear the buffer...
   */
 
-  memset(LineBuffers[row], 0, (size_t)header->cupsWidth * (size_t)DotBytes);
+  memset(LineBuffers[row], 0, header->cupsWidth * DotBytes);
 }
 
 
@@ -963,19 +983,16 @@ OutputRows(
  * 'main()' - Main entry and processing of driver.
  */
 
-int					/* O - Exit status */
-main(int  argc,				/* I - Number of command-line arguments */
-     char *argv[])			/* I - Command-line arguments */
+int				/* O - Exit status */
+main(int  argc,			/* I - Number of command-line arguments */
+     char *argv[])		/* I - Command-line arguments */
 {
-  int			fd;		/* File descriptor */
-  cups_raster_t		*ras;		/* Raster stream for printing */
-  cups_page_header2_t	header;		/* Page header from file */
-  ppd_file_t		*ppd;		/* PPD file */
-  int			page;		/* Current page */
-  unsigned		y;		/* Current line */
-#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
-  struct sigaction action;		/* Actions for POSIX signals */
-#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
+  int			fd;	/* File descriptor */
+  cups_raster_t		*ras;	/* Raster stream for printing */
+  cups_page_header_t	header;	/* Page header from file */
+  ppd_file_t		*ppd;	/* PPD file */
+  int			page;	/* Current page */
+  int			y;	/* Current line */
 
 
  /*
@@ -988,16 +1005,14 @@ main(int  argc,				/* I - Number of command-line arguments */
   * Check command-line...
   */
 
-  if (argc != 6 && argc != 7)
+  if (argc < 6 || argc > 7)
   {
    /*
     * We don't have the correct number of arguments; write an error message
     * and return.
     */
 
-    _cupsLangPrintFilter(stderr, "ERROR",
-                         _("%s job-id user title copies options [file]"),
-                         "rastertoepson");
+    fputs("ERROR: rastertoepson job-id user title copies options [file]\n", stderr);
     return (1);
   }
 
@@ -1009,7 +1024,7 @@ main(int  argc,				/* I - Number of command-line arguments */
   {
     if ((fd = open(argv[6], O_RDONLY)) == -1)
     {
-      _cupsLangPrintError("ERROR", _("Unable to open raster file"));
+      perror("ERROR: Unable to open raster file - ");
       sleep(1);
       return (1);
     }
@@ -1019,46 +1034,11 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   ras = cupsRasterOpen(fd, CUPS_RASTER_READ);
 
- /*
-  * Register a signal handler to eject the current page if the
-  * job is cancelled.
-  */
-
-  Canceled = 0;
-
-#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
-  sigset(SIGTERM, CancelJob);
-#elif defined(HAVE_SIGACTION)
-  memset(&action, 0, sizeof(action));
-
-  sigemptyset(&action.sa_mask);
-  action.sa_handler = CancelJob;
-  sigaction(SIGTERM, &action, NULL);
-#else
-  signal(SIGTERM, CancelJob);
-#endif /* HAVE_SIGSET */
-
  /*
   * Initialize the print device...
   */
 
   ppd = ppdOpenFile(getenv("PPD"));
-  if (!ppd)
-  {
-    ppd_status_t	status;		/* PPD error */
-    int			linenum;	/* Line number */
-
-    _cupsLangPrintFilter(stderr, "ERROR",
-                         _("The PPD file could not be opened."));
-
-    status = ppdLastError(&linenum);
-
-    fprintf(stderr, "DEBUG: %s on line %d.\n", ppdErrorString(status), linenum);
-
-    return (1);
-  }
-
-  Model = ppd->model_number;
 
   Setup();
 
@@ -1068,19 +1048,15 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   page = 0;
 
-  while (cupsRasterReadHeader2(ras, &header))
+  while (cupsRasterReadHeader(ras, &header))
   {
    /*
     * Write a status message with the page number and number of copies.
     */
 
-    if (Canceled)
-      break;
-
     page ++;
 
     fprintf(stderr, "PAGE: %d %d\n", page, header.NumCopies);
-    _cupsLangPrintFilter(stderr, "INFO", _("Starting page %d."), page);
 
    /*
     * Start the page...
@@ -1098,17 +1074,9 @@ main(int  argc,				/* I - Number of command-line arguments */
       * Let the user know how far we have progressed...
       */
 
-      if (Canceled)
-	break;
-
       if ((y & 127) == 0)
-      {
-        _cupsLangPrintFilter(stderr, "INFO",
-	                     _("Printing page %d, %u%% complete."),
-			     page, 100 * y / header.cupsHeight);
-        fprintf(stderr, "ATTR: job-media-progress=%u\n",
-		100 * y / header.cupsHeight);
-      }
+        fprintf(stderr, "INFO: Printing page %d, %d%% complete...\n", page,
+	        100 * y / header.cupsHeight);
 
      /*
       * Read a line of graphics...
@@ -1128,12 +1096,7 @@ main(int  argc,				/* I - Number of command-line arguments */
     * Eject the page...
     */
 
-    _cupsLangPrintFilter(stderr, "INFO", _("Finished page %d."), page);
-
     EndPage(&header);
-
-    if (Canceled)
-      break;
   }
 
  /*
@@ -1157,10 +1120,14 @@ main(int  argc,				/* I - Number of command-line arguments */
   */
 
   if (page == 0)
-  {
-    _cupsLangPrintFilter(stderr, "ERROR", _("No pages were found."));
-    return (1);
-  }
+    fputs("ERROR: No pages found!\n", stderr);
   else
-    return (0);
+    fputs("INFO: " CUPS_SVERSION " is ready to print.\n", stderr);
+
+  return (page == 0);
 }
+
+
+/*
+ * End of "$Id: rastertoepson.c,v 1.1.1.3 2002/03/02 18:28:31 jlovell Exp $".
+ */
